@@ -2,6 +2,9 @@
     begin                : Fri Sep 13 2002
     copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
+
+    copyright            : (C) 2009 by Michael Pyne
+    email                : michael.pyne@kdemail.net
  ***************************************************************************/
 
 /***************************************************************************
@@ -25,8 +28,9 @@
 
 #include <QEvent>
 #include <QVBoxLayout>
+#include <QLatin1String>
 #include <QList>
-#include <Q3WidgetStack>
+#include <QStackedWidget>
 #include <QSizePolicy>
 
 #include "searchwidget.h"
@@ -42,16 +46,17 @@
 // public methods
 ////////////////////////////////////////////////////////////////////////////////
 
-PlaylistSplitter::PlaylistSplitter(QWidget *parent, const char *name) :
+PlaylistSplitter::PlaylistSplitter(PlayerManager *player, QWidget *parent) :
     QSplitter(Qt::Horizontal, parent),
     m_newVisible(0),
     m_playlistBox(0),
     m_searchWidget(0),
     m_playlistStack(0),
     m_editor(0),
-    m_nowPlaying(0)
+    m_nowPlaying(0),
+    m_player(player)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String("playlistSplitter"));
 
     setupActions();
     setupLayout();
@@ -59,6 +64,9 @@ PlaylistSplitter::PlaylistSplitter(QWidget *parent, const char *name) :
 
     m_editor->slotUpdateCollection();
     m_editor->setupObservers();
+
+    connect(m_player, SIGNAL(signalItemChanged(FileHandle)),
+            this,     SIGNAL(playingItemChanged(FileHandle)));
 }
 
 PlaylistSplitter::~PlaylistSplitter()
@@ -170,13 +178,12 @@ void PlaylistSplitter::setupLayout()
     topLayout->setMargin(0);
     topLayout->setSpacing(0);
 
-    m_playlistStack = new Q3WidgetStack(top, "playlistStack");
+    m_playlistStack = new QStackedWidget(top);
+    m_playlistStack->setObjectName("playlistStack");
     m_playlistStack->installEventFilter(this);
     m_playlistStack->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     m_playlistStack->hide(); // Will be shown after CollectionList filled.
 
-    connect(m_playlistStack, SIGNAL(aboutToShow(QWidget *)), this, SLOT(slotPlaylistChanged(QWidget *)));
-
     m_editor = new TagEditor(editorSplitter);
     m_editor->setObjectName("TagEditor");
 
@@ -195,13 +202,19 @@ void PlaylistSplitter::setupLayout()
     connect(m_playlistBox, SIGNAL(signalPlaylistDestroyed(Playlist *)),
             m_editor, SLOT(slotPlaylistDestroyed(Playlist *)));
     connect(m_playlistBox, SIGNAL(startupComplete()), SLOT(slotEnable()));
+    connect(m_playlistBox, SIGNAL(startFilePlayback(FileHandle)),
+            m_player, SLOT(play(FileHandle)));
+
+    m_player->setPlaylistInterface(m_playlistBox);
+
+    // Let interested parties know we're ready
+    connect(m_playlistBox, SIGNAL(startupComplete()), SIGNAL(guiReady()));
 
     insertWidget(0, m_playlistBox);
 
-    connect(CollectionList::instance(), SIGNAL(signalCollectionChanged()),
-            m_editor, SLOT(slotUpdateCollection()));
-
     m_nowPlaying = new NowPlaying(top, m_playlistBox);
+    connect(m_player, SIGNAL(signalItemChanged(FileHandle)),
+            m_nowPlaying, SLOT(slotUpdate(FileHandle)));
 
     // Create the search widget -- this must be done after the CollectionList is created.
 
@@ -223,6 +236,11 @@ void PlaylistSplitter::setupLayout()
     topLayout->addWidget(m_searchWidget);
     topLayout->addWidget(m_playlistStack, 1);
 
+    // Now that GUI setup is complete, add some auto-update signals.
+    connect(CollectionList::instance(), SIGNAL(signalCollectionChanged()),
+            m_editor, SLOT(slotUpdateCollection()));
+    connect(m_playlistStack, SIGNAL(currentChanged(int)), this, SLOT(slotPlaylistChanged(int)));
+
     // Show the collection on startup.
     m_playlistBox->setSelected(0, true);
 }
@@ -263,9 +281,9 @@ void PlaylistSplitter::slotPlaylistSelectionChanged()
     m_editor->slotSetItems(visiblePlaylist()->selectedItems());
 }
 
-void PlaylistSplitter::slotPlaylistChanged(QWidget *w)
+void PlaylistSplitter::slotPlaylistChanged(int i)
 {
-    Playlist *p = dynamic_cast<Playlist *>(w);
+    Playlist *p = qobject_cast<Playlist *>(m_playlistStack->widget(i));
 
     if(!p)
         return;
