@@ -39,6 +39,7 @@
 #include <QMenu>
 #include <Q3MimeSourceFactory>
 #include <QLabel>
+#include <QPainter>
 #include <QVBoxLayout>
 #include <QMouseEvent>
 #include <QDesktopWidget>
@@ -58,8 +59,6 @@
 
 using namespace ActionCollection;
 
-static bool copyImage(QImage &dest, QImage &src, int x, int y);
-
 PassiveInfo::PassiveInfo(QSystemTrayIcon *parent) :
     QFrame(static_cast<QWidget *>(0), 
         Qt::ToolTip | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint
@@ -86,6 +85,7 @@ void PassiveInfo::startTimer(int delay)
 void PassiveInfo::show()
 {
     m_timer->start(3500);
+    setWindowOpacity(1.0);
     QFrame::show();
 }
 
@@ -174,10 +174,12 @@ void PassiveInfo::positionSelf()
 // public methods
 ////////////////////////////////////////////////////////////////////////////////
 
-SystemTray::SystemTray(QWidget *parent) : KSystemTrayIcon(parent),
+SystemTray::SystemTray(PlayerManager *player, QWidget *parent) : KSystemTrayIcon(parent),
                                           m_popup(0),
+                                          m_player(player),
                                           m_fadeTimer(0),
-                                          m_fade(true)
+                                          m_fade(true),
+                                          m_hasCompositionManager(false)
 
 {
     // This should be initialized to the number of labels that are used.
@@ -204,9 +206,9 @@ SystemTray::SystemTray(QWidget *parent) : KSystemTrayIcon(parent),
 
     QMenu *cm = contextMenu();
 
-    connect(PlayerManager::instance(), SIGNAL(signalPlay()), this, SLOT(slotPlay()));
-    connect(PlayerManager::instance(), SIGNAL(signalPause()), this, SLOT(slotPause()));
-    connect(PlayerManager::instance(), SIGNAL(signalStop()), this, SLOT(slotStop()));
+    connect(m_player, SIGNAL(signalPlay()), this, SLOT(slotPlay()));
+    connect(m_player, SIGNAL(signalPause()), this, SLOT(slotPause()));
+    connect(m_player, SIGNAL(signalStop()), this, SLOT(slotStop()));
 
     cm->addAction( action("play") );
     cm->addAction( action("pause") );
@@ -230,10 +232,11 @@ SystemTray::SystemTray(QWidget *parent) : KSystemTrayIcon(parent),
     m_fadeTimer = new QTimer(this);
     m_fadeTimer->setObjectName("systrayFadeTimer");
     connect(m_fadeTimer, SIGNAL(timeout()), SLOT(slotNextStep()));
+    connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), SLOT(slotActivated(QSystemTrayIcon::ActivationReason)));
 
-    if(PlayerManager::instance()->playing())
+    if(m_player->playing())
         slotPlay();
-    else if(PlayerManager::instance()->paused())
+    else if(m_player->paused())
         slotPause();
 }
 
@@ -248,13 +251,13 @@ SystemTray::~SystemTray()
 
 void SystemTray::slotPlay()
 {
-    if(!PlayerManager::instance()->playing())
+    if(!m_player->playing())
         return;
 
-    QPixmap cover = PlayerManager::instance()->playingFile().coverInfo()->pixmap(CoverInfo::Thumbnail);
+    QPixmap cover = m_player->playingFile().coverInfo()->pixmap(CoverInfo::Thumbnail);
 
     setIcon(m_playPix);
-    setToolTip(PlayerManager::instance()->playingString(), cover);
+    setToolTip(m_player->playingString(), cover);
     createPopup();
 }
 
@@ -268,10 +271,10 @@ void SystemTray::slotTogglePopup()
 
 void  SystemTray::slotPopupLargeCover()
 {
-    if(!PlayerManager::instance()->playing())
+    if(!m_player->playing())
         return;
 
-    FileHandle playingFile = PlayerManager::instance()->playingFile();
+    FileHandle playingFile = m_player->playingFile();
     playingFile.coverInfo()->popup();
 }
 
@@ -282,6 +285,7 @@ void SystemTray::slotStop()
 
     delete m_popup;
     m_popup = 0;
+    m_fadeTimer->stop();
 }
 
 void SystemTray::slotPopupDestroyed()
@@ -292,26 +296,31 @@ void SystemTray::slotPopupDestroyed()
 
 void SystemTray::slotNextStep()
 {
-    QColor result;
+    // Could happen I guess if the timeout event were queued while we're deleting m_popup
+    if(!m_popup)
+        return;
 
     ++m_step;
 
-    // If we're not fading, immediately show the labels
-    if(!m_fade)
-        m_step = STEPS;
-
-    result = interpolateColor(m_step);
-
-    for(int i = 0; i < m_labels.size() && m_labels[i]; ++i) {
-        QPalette palette;
-        palette.setColor(m_labels[i]->foregroundRole(), result);
-        m_labels[i]->setPalette(palette);
-    }
-
-    if(m_step == STEPS) {
+    // If we're not fading, immediately stop the fadeout
+    if(!m_fade || m_step == STEPS) {
         m_step = 0;
         m_fadeTimer->stop();
         emit fadeDone();
+        return;
+    }
+
+    if(m_hasCompositionManager) {
+        m_popup->setWindowOpacity((1.0 * STEPS - m_step) / STEPS);
+    }
+    else {
+        QColor result = interpolateColor(m_step);
+
+        for(int i = 0; i < m_labels.size() && m_labels[i]; ++i) {
+            QPalette palette;
+            palette.setColor(m_labels[i]->foregroundRole(), result);
+            m_labels[i]->setPalette(palette);
+        }
     }
 }
 
@@ -320,6 +329,12 @@ void SystemTray::slotFadeOut()
     m_startColor = m_labels[0]->palette().color( QPalette::Text ); //textColor();
     m_endColor = m_labels[0]->palette().color( QPalette::Window ); //backgroundColor();
 
+#ifdef Q_WS_X11
+    m_hasCompositionManager = QX11Info::isCompositingManagerRunning();
+#else
+    m_hasCompositionManager = true; // Assume yes for non-X11
+#endif
+
     connect(this, SIGNAL(fadeDone()), m_popup, SLOT(hide()));
     connect(m_popup, SIGNAL(mouseEntered()), this, SLOT(slotMouseInPopup()));
     m_fadeTimer->start(1500 / STEPS);
@@ -333,6 +348,9 @@ void SystemTray::slotMouseInPopup()
     m_endColor = m_labels[0]->palette().color( QPalette::Text ); //textColor();
     disconnect(SIGNAL(fadeDone()));
 
+    if(m_hasCompositionManager)
+        m_popup->setWindowOpacity(1.0);
+
     m_step = STEPS - 1; // Simulate end of fade to solid text
     slotNextStep();
 }
@@ -384,7 +402,7 @@ KVBox *SystemTray::createPopupLayout(QWidget *parent, const FileHandle &file)
 
 void SystemTray::createPopup()
 {
-    FileHandle playingFile = PlayerManager::instance()->playingFile();
+    FileHandle playingFile = m_player->playingFile();
     Tag *playingInfo = playingFile.tag();
 
     // If the action exists and it's checked, do our stuff
@@ -436,14 +454,6 @@ void SystemTray::createPopup()
         m_labels[labelCount++]->setText(s);
     }
 
-    m_startColor = m_labels[0]->palette().color( QPalette::Window ); //backgroundColor();
-    m_endColor = m_labels[0]->palette().color( QPalette::Text ); //textColor();
-    //m_startColor = m_labels[0]->backgroundColor();
-    //m_endColor = m_labels[0]->textColor();
-
-    slotNextStep();
-    m_fadeTimer->start(1500 / STEPS);
-
     m_popup->setView(box);
     m_popup->show();
 }
@@ -463,8 +473,17 @@ QPixmap SystemTray::createPixmap(const QString &pixName)
     QImage fgImage = SmallIcon(pixName).toImage(); // Should be 16x16
 
     KIconEffect::semiTransparent(bgImage);
-    copyImage(bgImage, fgImage, (bgImage.width() - fgImage.width()) / 2,
-              (bgImage.height() - fgImage.height()) / 2);
+
+    // Center smaller image inside larger one.
+    QRect rect = fgImage.rect();
+    rect.moveCenter(bgImage.rect().center());
+
+    // Must use begin()/end() (or scope in this case) to avoid returning a
+    // QPixmap constructed from a QImage being painted on.
+    {
+        QPainter painter(&bgImage);
+        painter.drawImage(rect.topLeft(), fgImage);
+    }
 
     return QPixmap::fromImage(bgImage);
 }
@@ -612,53 +631,6 @@ void SystemTray::slotActivated(QSystemTrayIcon::ActivationReason reason)
         action("play")->trigger();
 }
 
-/*
- * This function copies the entirety of src into dest, starting in
- * dest at x and y.  This function exists because I was unable to find
- * a function like it in either QImage or kdefx
- */
-
-static bool copyImage(QImage &dest, QImage &src, int x, int y)
-{
-    if(dest.depth() != src.depth())
-        return false;
-    if((x + src.width()) >= dest.width())
-        return false;
-    if((y + src.height()) >= dest.height())
-        return false;
-
-    // We want to use KIconEffect::overlay to do this, since it handles
-    // alpha, but the images need to be the same size.  We can handle that.
-
-    QImage large_src(dest);
-
-    // It would perhaps be better to create large_src based on a size, but
-    // this is the easiest way to make a new image with the same depth, size,
-    // etc.
-
-    large_src.detach();
-
-    // However, we do have to specifically ensure that setAlphaBuffer is set
-    // to false
-
-    large_src = large_src.convertToFormat(QImage::Format_RGB32); // Turn off alpha
-    large_src.fill(0); // All transparent pixels
-    large_src = large_src.convertToFormat(QImage::Format_ARGB32); // Turn on alpha
-
-    int w = src.width();
-    int h = src.height();
-    for(int dx = 0; dx < w; dx++)
-        for(int dy = 0; dy < h; dy++)
-            large_src.setPixel(dx + x, dy + y, src.pixel(dx, dy));
-
-    // Apply effect to image
-
-    KIconEffect::overlay(dest, large_src);
-
-    return true;
-}
-
-
 #include "systemtray.moc"
 
 // vim: set et sw=4 tw=0 sta:
